#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-2-Clause
#
# Copyright 2024 NXP
#

import json

modes = {'encrypt': 0, 'decrypt': 1}


def to_compound_str(name, val):
    assert len(val) % 2 == 0, "Only even sized values supported"
    if len(val) > 0:
        import re
        a = re.findall('..', val)
        b = name + " = (const uint8_t []){ "
        for s in a:
            b += "0x" + s + ", "
        b += "},\n\t" + name + "_len = " + repr((int)(len(val) / 2)) + ","
    else:
        b = name + " = NULL,\n\t" + name + "_len = 0,"
    return b


def generate_case(outf, tv, mode):
    outf.write('{\n\t.algo = TEE_ALG_AES_GCM, .mode = ' + mode +
               ', .key_type = TEE_TYPE_AES,\n')
    outf.write('\t' + to_compound_str('.key', tv['key']) + '\n')
    outf.write('\t' + to_compound_str('.nonce', tv['iv']) + '\n')
    outf.write('\t.aad_incr = 0,\n')
    outf.write('\t' + to_compound_str('.aad', tv['aad']) + '\n')
    outf.write('\t.in_incr = 0,\n')
    outf.write('\t' + to_compound_str('.ptx', tv['msg']) + '\n')
    outf.write('\t' + to_compound_str('.ctx', tv['ct']) + '\n')
    outf.write('\t' + to_compound_str('.tag', tv['tag']) + '\n')
    outf.write('\t.line = __LINE__,\n')
    outf.write('\t.id = ' + repr(tv['tcId']) + '\n},\n')


def get_args():
    import argparse

    parser = argparse.ArgumentParser()

    parser.add_argument('--inf', required=True,
                        type=argparse.FileType('r'),
                        help='Name of input json file')

    parser.add_argument('--outf', required=True,
                        type=argparse.FileType('w'),
                        help='Name of output C file')

    parser.add_argument('--mode', required=True, choices=modes.keys(),
                        help='mode: encrypt or decrypt')

    return parser.parse_args()


# Convert google/wycheproof AES GCM test vectors to xtest AE test cases
def main():
    args = get_args()
    inf = args.inf
    outf = args.outf

    outf.write("/* SPDX-License-Identifier: Apache-2.0 */\n")
    outf.write("/*\n")
    outf.write(" * Copyright 2024 NXP\n")
    outf.write(" */\n\n")

    if args.mode == "encrypt":
        mode = "TEE_MODE_ENCRYPT"
    else:
        mode = "TEE_MODE_DECRYPT"

    data = json.load(inf)

    for tg in data['testGroups']:
        for tv in tg['tests']:
            if tv['result'] == 'valid' and 'CounterWrap' in tv['flags']:
                generate_case(outf, tv, mode)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-2-Clause
#
# Copyright (c) 2018, Linaro Limited
#

def get_args():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('--name', required=True,
                        help='Name of variable in C file')
    parser.add_argument('--out', required=True, help='Name of C file')
    parser.add_argument('--inf', required=True, help='Name of in file')

    return parser.parse_args()

def main():
    import array

    args = get_args()

    inf = open(args.inf, 'r')
    f = open(args.out, 'w')

    f.write("/* automatically generated */\n")
    f.write("#include <stdint.h>\n")
    f.write("#include <stddef.h>\n\n")

    f.write("const uint8_t " + args.name + "[] = {\n")
    i = 0
    for x in array.array("B", map(ord, (inf.read()))):
        f.write("0x" + '{0:02x}'.format(x) + ",")
        i = i + 1
        if i % 8 == 0:
            f.write("\n")
        else:
            f.write(" ")

    f.write("'\\0'};\n")
    f.write("const size_t " + args.name + "_size = sizeof(" + 
            args.name + ") - 1;\n")

    f.close()
    inf.close()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-2-Clause
#
# Copyright (c) 2017, Linaro Limited
#

modes = {'encrypt': 0, 'decrypt': 1}

limited = False


def to_compound_str(val):
    assert len(val) % 2 == 0, "Only even sized values supported"
    if len(val) > 0:
        import re
        a = re.findall('..', val)
        b = "(const uint8_t []){"
        for s in a:
            b += "0x" + s + ", "
        b += "}, " + repr(len(val) / 2) + ","
    else:
        b = "NULL, 0,"
    return b


def generate_case(outf, myvars, mode):
    if 'PT' not in myvars:
        myvars['PT'] = ''
    if 'FAIL' in myvars:
        return
    if limited and myvars['Count'] != '0':
        return
    # Skip cases not supported by GP
    if len(myvars['Tag']) / 2 < 96 / 8:
        return

    outf.write('{ TEE_ALG_AES_GCM, ' + mode + ', TEE_TYPE_AES,\n')
    outf.write('/* Key */ ' + to_compound_str(myvars['Key']) + '\n')
    outf.write('/* IV  */ ' + to_compound_str(myvars['IV']) + '\n')
    outf.write('0,\n')
    outf.write('/* AAD */ ' + to_compound_str(myvars['AAD']) + '\n')
    outf.write('0,\n')
    outf.write('/* PT  */ ' + to_compound_str(myvars['PT']) + '\n')
    outf.write('/* CT  */ ' + to_compound_str(myvars['CT']) + '\n')
    outf.write('/* Tag */ ' + to_compound_str(myvars['Tag']) + '\n')
    outf.write(repr(myvars['Line']) + '},\n')


def get_args():
    import argparse

    parser = argparse.ArgumentParser()

    parser.add_argument('--inf', required=True,
                        type=argparse.FileType('r'),
                        help='Name of input RSP file')

    parser.add_argument('--outf', required=True,
                        type=argparse.FileType('w'),
                        help='Name of output C file')

    parser.add_argument('--mode', required=True, choices=modes.keys(),
                        help='mode: encrypt or decrypt')

    parser.add_argument('--limited', action="store_true",
                        help='Only run one test case from each group')

    return parser.parse_args()


def main():
    import re
    global limited
    args = get_args()
    inf = args.inf
    outf = args.outf
    myvars = {}
    line_num = 0

    if args.mode == "encrypt":
        mode = "TEE_MODE_ENCRYPT"
    else:
        mode = "TEE_MODE_DECRYPT"

    limited = args.limited

    for line in inf:
        line_num += 1
        myl = line.strip()
        if len(myl) == 0:
            continue
        if re.match('^#', myl):
            continue
        if re.match('^\[', myl):
            continue
        s = re.split('\W+', myl)
        if len(s) == 0:
            continue
        name = s[0]
        if name == 'Count':
            if len(myvars) > 1:
                generate_case(outf, myvars, mode)
                myvars = {}
            myvars['Line'] = line_num

        if len(s) < 2:
            myvars[s[0]] = ''
        else:
            myvars[s[0]] = s[1]

        if len(s) < 2:
            continue
        val = s[1]

    if len(myvars) > 1:
        generate_case(outf, myvars, mode)


if __name__ == "__main__":
    main()
